@@ -2,10 +2,8 @@
 # Myokit symbolic expression classes. Defines different expressions, equations
 # and the unit system.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
@@ -59,8 +57,10 @@ class Expression(object):
 
     def __init__(self, operands=None):
         self._token = None
+
         # Store operands
         self._operands = () if operands is None else operands
+
         # Cached values
         # These _could_ be calculated immediatly, since they won't change in
         # the object's lifetime. However, it turns out to be faster to only
@@ -68,6 +68,7 @@ class Expression(object):
         self._cached_hash = None
         self._cached_polish = None
         self._cached_validation = None
+
         # Store references
         self._references = set()
         for op in self._operands:
@@ -136,6 +137,9 @@ class Expression(object):
         """
         raise NotImplementedError
 
+    def __contains__(self, key):
+        return key in self._operands
+
     def contains_type(self, kind):
         """
         Returns True if this expression tree contains an expression of the
@@ -283,7 +287,9 @@ class Expression(object):
         try:
             return self._eval_unit(mode)
         except EvalUnitError as e:
-            raise myokit.IncompatibleUnitError(_expr_error_message(self, e))
+            message = _expr_error_message(self, e)
+            token = e.expr._token
+        raise myokit.IncompatibleUnitError(message, token)
 
     def _eval_unit(self, mode):
         """
@@ -446,6 +452,9 @@ class Expression(object):
         """
         return set(self._references)
 
+    def __repr__(self):
+        return 'myokit.Expression[' + self.code() + ']'
+
     def __str__(self):
         return self.code()
 
@@ -713,11 +722,6 @@ class Name(LhsExpression):
 
     def __init__(self, value):
         super(Name, self).__init__()
-        if not isinstance(value, myokit.Variable):
-            if not isinstance(value, basestring):
-                raise ValueError(
-                    'myokit.Name objects must have a value that is a'
-                    ' myokit.Variable (or, when debugging, a string).')
         self._value = value
         self._references = set([self])
 
@@ -740,11 +744,7 @@ class Name(LhsExpression):
         return Name(self._value)
 
     def _code(self, b, c):
-        if isinstance(self._value, basestring):
-            # Allow an exception for strings (used in function templates and
-            # debugging).
-            b.write('str:' + str(self._value))
-        else:
+        if isinstance(self._value, myokit.Variable):
             if self._value.is_nested():
                 b.write(self._value.name())
             else:
@@ -755,6 +755,12 @@ class Name(LhsExpression):
                     except KeyError:
                         pass
                 b.write(self._value.qname(c))
+        elif isinstance(self._value, basestring):
+            # Allow strings for debugging
+            b.write('str:' + str(self._value))
+        else:
+            # And sneaky abuse of the expression system
+            b.write(str(self._value))
 
     def _eval_unit(self, mode):
 
@@ -1092,7 +1098,7 @@ class Plus(InfixExpression):
 
         raise EvalUnitError(
             self, 'Addition requires equal units, got '
-            + str(unit1) + ' and ' + str(unit2) + '.')
+            + unit1.clarify() + ' and ' + unit2.clarify() + '.')
 
 
 class Minus(InfixExpression):
@@ -1130,8 +1136,8 @@ class Minus(InfixExpression):
             return unit1
 
         raise EvalUnitError(
-            self, 'Subtraction requires equal units, got ' + str(unit1)
-            + ' and ' + str(unit2) + '.')
+            self, 'Subtraction requires equal units, got '
+            + unit1.clarify() + ' and ' + unit2.clarify() + '.')
 
 
 class Multiply(InfixExpression):
@@ -1336,7 +1342,11 @@ class Power(InfixExpression):
 
         if unit1 is None:
             return None
-        return unit1 ** self._op2.eval()
+
+        try:
+            return unit1 ** self._op2.eval()
+        except TypeError as e:
+            raise EvalUnitError(self, str(e))
 
 
 class Function(Expression):
@@ -1428,21 +1438,6 @@ class UnaryDimensionlessFunction(Function):
         return myokit.units.dimensionless
 
 
-class UnsupportedFunction(Function):
-    """
-    Unsupported functions in other formats than myokit can be imported as an
-    ``UnsupportedFunction``. This preserves the meaning of the original
-    document. UnsupportedFunction objects should never occur in valid models.
-    """
-    def __init__(self, name, ops):
-        self._nargs = [len(ops)]
-        self._fname = 'UNSUPPORTED::' + str(name).strip()
-        super(UnsupportedFunction, self).__init__(*ops)
-
-    def _validate(self, trail):
-        raise IntegrityError('UnsupportedFunction in expression.', self._token)
-
-
 class Sqrt(Function):
     """
     Represents the square root ``sqrt(x)``.
@@ -1468,8 +1463,8 @@ class Sqrt(Function):
             return None
         try:
             return unit ** 0.5
-        except ValueError as e:
-            raise EvalUnitError(self, e)
+        except TypeError as e:
+            raise EvalUnitError(self, str(e))
 
 
 class Sin(UnaryDimensionlessFunction):
@@ -2302,6 +2297,10 @@ class EvalUnitError(Exception):
     def __init__(self, expr, err):
         self.expr = expr
         self.err = err
+        self.line = self.char = None
+        if expr._token is not None:
+            self.line = expr._token[2]
+            self.char = expr._token[3]
 
 
 def _expr_error_message(owner, e):
@@ -2337,7 +2336,14 @@ def _expr_error_message(owner, e):
 
     # Show there was an error
     out = []
-    out.append(str(e.err))
+    if isinstance(e, EvalUnitError):
+        msg = 'Incompatible units'
+        if e.expr._token is not None:
+            msg += ' on line ' + str(e.expr._token[2])
+        msg += ': ' + str(e.err)
+        out.append(msg)
+    else:
+        out.append(str(e.err))
     out.append('Encountered when evaluating')
     par_str = '  ' + owner.code()
     err_str = e.expr.code()
@@ -2463,44 +2469,142 @@ class Unit(object):
         """
         return unit1._x == unit2._x
 
-    @staticmethod
-    def conversion_factor(unit1, unit2):
+    def clarify(self):
         """
-        Returns the number ``c`` such that ``1 [unit1] = c [unit2]``.
+        Returns a string showing this unit's representation using both the
+        short syntax of ``str(unit)``, and the long syntax of ``repr(unit)``.
+
+        For example::
+
+            >>> from myokit import Unit
+            >>> print(str(myokit.units.katal))
+            [kat]
+            >>> print(repr(myokit.units.katal))
+            [mol/s]
+            >>> print(myokit.units.katal.clarify())
+            [kat] (or [mol/s])
+
+        If both representations are equal, the second part is omitted""
+
+            >>> print(myokit.units.m.clarify())
+            [m]
+
+        """
+        r1 = str(self)
+        r2 = repr(self)
+        if r1 == r2:
+            return r1
+        return r1 + ' (or ' + r2 + ')'
+
+    @staticmethod
+    def conversion_factor(unit1, unit2, helpers=None):
+        """
+        Returns a :class:`myokit.Quantity` ``c`` to convert from ``unit1`` to
+        ``unit2``, such that ``1 [unit1] * c = 1 [unit2]``.
+
+        For example:
 
             >>> import myokit
             >>> myokit.Unit.conversion_factor('m', 'km')
-            0.001
+            0.001 [1 (1000)]
 
+        Where::
+
+            1 [m] = 0.001 [km/m] * 1 [km]
+
+        so that ``c = 0.001 [km/m]``, and the unit ``[km/m]`` can be written as
+        ``[km/m] = [ kilo ] = [1 (1000)]``.
+
+        Conversions between incompatible units can be performed if one or
+        multiple helper :class:`Quantity` objects are passed in.
+
+        For example:
+
+            >>> import myokit
+            >>> myokit.Unit.conversion_factor(
+            ...     'uA/cm^2', 'uA/uF', ['1 [uF/cm^2]'])
+            1.0 [cm^2/uF]
+
+        Where::
+
+            1 [uA/cm^2] = 1 [cm^2/uF] * 1 [uA/uF]
+
+        Arguments:
+
+        ``unit1``
+            The new unit to convert from, given as a :class:`myokit.Unit` or as
+            a string that can be converted to a unit with
+            :meth:`myokit.parse_unit()`.
+        ``unit2``
+            The new unit to convert to.
+        ``helpers=None``
+            An optional list of conversion factors, which the method will
+            attempt to use if the new and old units are incompatible. Each
+            factor should be specified as a :class:`myokit.Quantity` or
+            something that can be converted to a Quantity e.g. a string
+            ``1 [uF/cm^2]``.
+
+        Returns a :class:`myokit.Quantity`.
         """
+        # Check unit1
         if not isinstance(unit1, myokit.Unit):
             if unit1 is None:
                 unit1 = myokit.units.dimensionless
             else:
-                try:
-                    unit1 = myokit.parse_unit(unit1)
-                except Exception:
-                    raise myokit.IncompatibleUnitError(
-                        'Cannot convert given object ' + repr(unit1)
-                        + ' to unit.')
+                unit1 = myokit.parse_unit(unit1)
 
+        # Check unit2
         if not isinstance(unit2, myokit.Unit):
             if unit2 is None:
                 unit2 = myokit.units.dimensionless
             else:
-                try:
-                    unit2 = myokit.parse_unit(unit2)
-                except Exception:
-                    raise myokit.IncompatibleUnitError(
-                        'Cannot convert given object ' + repr(unit2)
-                        + ' to unit.')
+                unit2 = myokit.parse_unit(unit2)
 
-        if unit1._x != unit2._x:
-            raise myokit.IncompatibleUnitError(
-                'Cannot convert from ' + str(unit1) + ' to ' + str(unit2)
-                + '.')
+        # Check helper units
+        factors = []
+        if helpers is not None:
+            for factor in helpers:
+                if not isinstance(factor, myokit.Quantity):
+                    factor = myokit.Quantity(factor)
+                factors.append(factor)
+        del(helpers)
 
-        return 1 if unit1._m == unit2._m else 10**(unit1._m - unit2._m)
+        # Simplest case: units are equal
+        if unit1 == unit2:
+            return Quantity(1)
+
+        # Get conversion factor
+        fw = None
+        if unit1._x == unit2._x:
+
+            # Directly convertible
+            fw = 10**(unit1._m - unit2._m)
+
+        else:
+            # Try conversion via one of the helpers
+            for factor in factors:
+
+                unit1a = unit1 * factor.unit()
+                if unit1a._x == unit2._x:
+                    fw = 10**(unit1a._m - unit2._m) * factor.value()
+                    break
+
+                unit1a = unit1 / factor.unit()
+                if unit1a._x == unit2._x:
+                    fw = 10**(unit1a._m - unit2._m) / factor.value()
+                    break
+
+        # Unable to convert?
+        if fw is None:
+            msg = 'Unable to convert from ' + unit1.clarify()
+            msg += ' to ' + unit2.clarify()
+            if factors:
+                msg += ' (even with help of conversion factors).'
+            raise myokit.IncompatibleUnitError(msg + '.')
+
+        # Create Quantity and return
+        fw = myokit._round_if_int(fw)
+        return Quantity(fw, unit2 / unit1)
 
     @staticmethod
     def convert(amount, unit1, unit2):
@@ -2513,17 +2617,13 @@ class Unit(object):
             3.0
 
         """
-        return amount * Unit.conversion_factor(unit1, unit2)
+        factor = Unit.conversion_factor(unit1, unit2)
+        if isinstance(amount, myokit.Quantity):
+            return factor * amount
+        return factor.value() * amount
 
-    def __div__(self, other):
-        """
-        Evaluates ``self / other``
-        """
-        if not isinstance(other, Unit):
-            return Unit(list(self._x), self._m - math.log10(float(other)))
-        return Unit(
-            [a - b for a, b in zip(self._x, other._x)],
-            self._m - other._m)
+    def __div__(self, other):   # pragma: no cover      truediv used instead
+        return self.__truediv__(other)
 
     def __eq__(self, other):
         if not isinstance(other, Unit):
@@ -2537,9 +2637,8 @@ class Unit(object):
         return list(self._x)
 
     def __float__(self):
-        """
-        Attempts to convert this unit to a float.
-        """
+        # Attempts to convert this unit to a float.
+
         for x in self._x:
             if x != 0:
                 raise TypeError(
@@ -2547,9 +2646,8 @@ class Unit(object):
         return self.multiplier()
 
     def __hash__(self):
-        """
-        Creates a hash for this unit
-        """
+        # Creates a hash for this Unit
+
         if not self._hash:
             self._hash = hash(
                 ','.join([str(x) for x in self._x]) + 'e' + str(self._m))
@@ -2575,10 +2673,15 @@ class Unit(object):
         """
         return 10 ** self._m
 
+    def multiplier_log_10(self):
+        """
+        Returns the base 10 logarithm of this unit's multiplier.
+        """
+        return self._m
+
     def __mul__(self, other):
-        """
-        Evaluates ``self * other``
-        """
+        # Evaluates ``self * other``
+
         if not isinstance(other, Unit):
             return Unit(list(self._x), self._m + math.log10(float(other)))
         return Unit(
@@ -2652,23 +2755,19 @@ class Unit(object):
                 raise KeyError('Unknown unit: "' + str(name) + '".')
 
     def __pow__(self, f):
-        """
-        Evaluates ``self ^ other``
-        """
+        # Evaluates ``self ^ other``
+
         f = float(f)
-        e = [f * x for x in self._x]
+        e = [myokit._round_if_int(f * x) for x in self._x]
         for x in e:
-            if abs(float(x) - int(x)) > 1e-15:
-                raise ValueError(
+            if not x == int(x):
+                raise TypeError(
                     'Unit exponentiation (' + repr(self) + ') ^ '
                     + str(f) + ' failed: would result in non-integer'
                     + ' exponents, which is not supported.')
         return Unit([int(x) for x in e], self._m * f)
 
     def __rdiv__(self, other):  # pragma: no cover    rtruediv used instead
-        """
-        Evaluates ``other / self``, where other is not a unit
-        """
         return self.__rtruediv__(other)
 
     @staticmethod
@@ -2735,8 +2834,10 @@ class Unit(object):
         Returns this unit formatted in the base SI units.
         """
         if not self._repr:
+
             # SI unit names
             si = ['g', 'm', 's', 'A', 'K', 'cd', 'mol']
+
             # Get unit parts
             pos = []
             neg = []
@@ -2753,26 +2854,30 @@ class Unit(object):
             u = '*'.join(pos) if pos else '1'
             for x in neg:
                 u += '/' + str(x)
+
             # Add conversion factor
             if self._m != 0:
                 m = 10**self._m
-                if abs(m - int(m)) < 1e-15:
+                if myokit._feq(m, int(m)):
                     m = int(m)
-                u += ' (' + str(m) + ')'
+                if m < 1e6:
+                    m = str(m)
+                else:
+                    m = '{:<1.0e}'.format(m)
+                u += ' (' + m + ')'
             self._repr = '[' + u + ']'
+
         return self._repr
 
     def __rmul__(self, other):
-        """
-        Evaluates ``other * self``, where other is not a Unit
-        """
+        # Evaluates ``other * self``, where other is not a Unit
+
         return Unit(list(self._x), self._m + math.log10(other))
 
     def __rtruediv__(self, other):
-        """
-        Evaluates ``other / self``, where other is not a unit when future
-        division is active.
-        """
+        # Evaluates ``other / self``, where other is not a unit when future
+        # division is active.
+
         return Unit([-a for a in self._x], math.log10(other) - self._m)
 
     def __str__(self):
@@ -2788,9 +2893,13 @@ class Unit(object):
                 u = Unit(list(self._x), m)
                 rep = Unit._preferred_representations[u]
                 m = 10**(self._m - m)
-                if abs(m - int(m)) < 1e-15:
+                if m >= 1 and myokit._feq(m, int(m)):
                     m = int(m)
-                rep = '[' + rep + ' (' + str(m) + ')]'
+                if m < 1e6:
+                    m = str(m)
+                else:
+                    m = '{:<1.0e}'.format(m)
+                rep = '[' + rep + ' (' + m + ')]'
                 Unit._preferred_representations[self] = rep[1:-1]
                 return rep
             except KeyError:
@@ -2800,11 +2909,13 @@ class Unit(object):
                 return rep
 
     def __truediv__(self, other):
-        """
-        Evaluates self / other if future division is active.
-        """
-        # Only truediv is supported, so methods are equal
-        return self.__div__(other)
+        # Evaluates self / other if future division is active.
+
+        if not isinstance(other, Unit):
+            return Unit(list(self._x), self._m - math.log10(float(other)))
+        return Unit(
+            [a - b for a, b in zip(self._x, other._x)],
+            self._m - other._m)
 
 
 # Dimensionless unit, used to compare against
@@ -2817,7 +2928,7 @@ dimensionless = Unit()
 class Quantity(object):
     """
     Represents a quantity with a :class:`unit <myokit.Unit>`. Can be used to
-    perform unit-safe arithmetic with Myokit.
+    perform unit-safe arithmetic.
 
     Example::
 
@@ -2937,8 +3048,8 @@ class Quantity(object):
             other = Quantity(other)
         if self._unit != other._unit:
             raise myokit.IncompatibleUnitError(
-                'Cannot add quantities with units ' + str(self._unit)
-                + ' and ' + str(other._unit) + '.')
+                'Cannot add quantities with units ' + self._unit.clarify()
+                + ' and ' + other._unit.clarify() + '.')
         return Quantity(self._value + other._value, self._unit)
 
     def cast(self, unit):
@@ -2959,10 +3070,8 @@ class Quantity(object):
             unit = myokit.parse_unit(unit)
         return Quantity(self._value, unit)
 
-    def __div__(self, other):
-        if not isinstance(other, Quantity):
-            other = Quantity(other)
-        return Quantity(self._value / other._value, self._unit / other._unit)
+    def __div__(self, other):   # pragma: no cover      truediv used instead
+        return self.__truediv__(other)
 
     def __eq__(self, other):
         if not isinstance(other, Quantity):
@@ -2980,11 +3089,22 @@ class Quantity(object):
             other = Quantity(other)
         return Quantity(self._value * other._value, self._unit * other._unit)
 
+    def __pow__(self, f):
+        if isinstance(f, Quantity):
+            if f.unit() != myokit.units.dimensionless:
+                raise myokit.IncompatibleUnitError(
+                    'Exponent of power must be dimensionless')
+            f = f.value()
+        return Quantity(self._value ** f, self._unit ** f)
+
     def __radd__(self, other):
         return self + other
 
+    def __repr__(self):
+        return self._str
+
     def __rdiv__(self, other):  # pragma: no cover    rtruediv used instead
-        return Quanity(other) / self
+        return Quantity(other) / self
 
     def __rmul__(self, other):
         return self * other
@@ -2992,6 +3112,9 @@ class Quantity(object):
     def __rsub__(self, other):
         return Quantity(other) - self
 
+    def __rtruediv__(self, other):
+        return Quantity(other) / self
+
     def __str__(self):
         return self._str
 
@@ -3000,13 +3123,16 @@ class Quantity(object):
             other = Quantity(other)
         if self._unit != other._unit:
             raise myokit.IncompatibleUnitError(
-                'Cannot subtract quantities with units ' + str(self._unit)
-                + ' and ' + str(other._unit) + '.')
+                'Cannot subtract quantities with units ' + self._unit.clarify()
+                + ' and ' + other._unit.clarify() + '.')
         return Quantity(self._value - other._value, self._unit)
 
     def __truediv__(self, other):
-        # Only truediv is supported, so behaviour is identical to div
-        return self.__div__(other)
+        # Evaluates self / other
+
+        if not isinstance(other, Quantity):
+            other = Quantity(other)
+        return Quantity(self._value / other._value, self._unit / other._unit)
 
     def unit(self):
         """
@@ -3019,3 +3145,4 @@ class Quantity(object):
         Returns this Quantity's unitless value.
         """
         return self._value
+
