@@ -2,10 +2,8 @@
 # Creates common graphs
 # Uses matplotlib
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
@@ -336,8 +334,8 @@ def cumulative_current(
             colors.extend(colors)
     else:
         # Colormap
-        cmap = matplotlib.cm.get_cmap(name='Spectral')
-        colors = [cmap(i) for i in np.linspace(0.9, 0.1, len(currents))]
+        cmap = matplotlib.cm.get_cmap(name='tab20')
+        colors = [cmap(i) for i in range(len(currents))]
 
     # Plot
     op = on = 0
